package ch.zhaw.pm2.racetrack.strategy;

import ch.zhaw.pm2.racetrack.PositionVector;
import ch.zhaw.pm2.racetrack.PositionVector.Direction;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * The PathFollowerMoveStrategy class determines the next move based on a file containing points on a path.
 */
public class PathFollowerMoveStrategy implements MoveStrategy {
    private PositionVector currentPosition;
    private PositionVector currentVelocity;
    private ArrayList<PositionVector> pointList;
    private int pointer;


    public PathFollowerMoveStrategy(String path, PositionVector startPosition) throws FileNotFoundException {
        pointList = new ArrayList<>();
        pointer = 0;
        readFile(new File(path));
        currentPosition = startPosition;
        currentVelocity = new PositionVector(0, 0);
    }

    public void readFile(File trackFile) throws FileNotFoundException {
        Scanner scanner = new Scanner(new FileInputStream(trackFile), "UTF-8");
        while (scanner.hasNextLine()) {
            String line = scanner.nextLine();
            String[] coordinates = line.split("(\\(X:|, Y:|\\))");
            pointList.add(new PositionVector(Integer.parseInt(coordinates[1]), Integer.parseInt(coordinates[2])));
        }
    }

    @Override
    public Direction nextMove() {
        if (pointer >= pointList.size()) {
            return null;
        }
        int accelerationX = 0;
        int accelerationY = 0;
        if (pointList.get(pointer).equals(currentPosition)) {
            pointer ++;
        }


        if(currentVelocity.getX() > 0){
            accelerationX = -1;
        } else if(currentVelocity.getX() < 0) {
            accelerationX = 1;
        }
        if(currentVelocity.getY() > 0){
            accelerationY = -1;
        } else if(currentVelocity.getY() < 0) {
            accelerationY = 1;
        }


        PositionVector movementVector = new PositionVector(pointList.get(pointer).getX() - currentPosition.getX(), pointList.get(pointer).getY() - currentPosition.getY());
        if(movementVector.getX() > 0 && movementVector.getX()/2.0 > currentVelocity.getX()) {
            accelerationX = 1;
        }
        else if(movementVector.getX() > 0 && movementVector.getX()/2.0 <= currentVelocity.getX()) {
            accelerationX = -1;
        }
        else if(movementVector.getX() < 0 && movementVector.getX()/2.0 < currentVelocity.getX()) {
            accelerationX = -1;
        }
        else if(movementVector.getX() < 0 && movementVector.getX()/2.0 >= currentVelocity.getX()) {
            accelerationX = 1;
        }

        if(movementVector.getY() > 0 && movementVector.getY()/2.0 > currentVelocity.getY()) {
            accelerationY = 1;
        }
        else if(movementVector.getY() > 0 && movementVector.getY()/2.0 <= currentVelocity.getY()) {
            accelerationY = -1;
        }
        else if(movementVector.getY() < 0 && movementVector.getY()/2.0 < currentVelocity.getY()) {
            accelerationY = -1;
        }
        else if(movementVector.getY() < 0 && movementVector.getY()/2.0 >= currentVelocity.getY()) {
            accelerationY = 1;
        }


        currentVelocity = new PositionVector(currentVelocity.getX() + accelerationX, currentVelocity.getY() + accelerationY);
        currentPosition = new PositionVector(currentPosition.getX() + currentVelocity.getX(), currentPosition.getY() + currentVelocity.getY());

        PositionVector acceleration = new PositionVector(accelerationX, accelerationY);
        Direction[] directions = Direction.values();
        for (Direction direction : directions) {
            if (direction.vector.equals(acceleration)) {
                return direction;
            }
        }
    return null;

    }
}
